/*
 * Dialog.c
 */

#include "stratego.h"

#define CHAR_WIDTH(f)	((f)->max_bounds.rbearing - (f)->min_bounds.lbearing -2)
#define CHAR_HEIGHT(f)	((f)->max_bounds.ascent + (f)->max_bounds.descent - 2)

#define max(a, b)	((a) > (b) ? (a) : (b))

#define DIALOG_OK		0
#define DIALOG_CANCEL		1
#define DIALOG_UNDECIDED	2

static int dialog_status;

static XtCallbackProc DialogOk ()
{
	dialog_status = DIALOG_OK;
}

static XtCallbackProc DialogCancel ()
{
	dialog_status = DIALOG_CANCEL;
}

Bool DialogBoxResult (prompt, input_buffer, edit_length, cancel_fl)
char *prompt, *input_buffer;
int edit_length, cancel_fl;
{
	Widget Popupshell, Dialog, Editfield;
	Dimension width, twidth, theight, dwidth;
	XFontStruct *font;
	XEvent event;
	Dimension right_margin, left_margin;
	Position x, y;
	Arg args[3];
	int argc;

	Popupshell = XtCreatePopupShell ("PopupShell", overrideShellWidgetClass,
					 Toplevel, NULL, ZERO);

	XtSetArg (args[0], XtNlabel, prompt);
	argc = 1;

	if (input_buffer != NULL) {
		if (strlen (input_buffer) > edit_length)
			edit_length = strlen (input_buffer);
		XtSetArg (args[1], XtNvalue, input_buffer);
		argc = 2;
	}
	Dialog = XtCreateManagedWidget ("Popup", dialogWidgetClass,
					Popupshell, args, argc);

	XawDialogAddButton (Dialog, "   OK   ", (XtCallbackProc)DialogOk, NULL);
	if (cancel_fl == ADD_CANCEL)
		XawDialogAddButton (Dialog, " CANCEL ", (XtCallbackProc)DialogCancel, NULL);

	font = NULL;
	XtSetArg (args[0], XtNfont, &font);
	XtGetValues (Dialog, args, 1);
	if (font != NULL)
		dwidth = CHAR_WIDTH (font) * strlen(prompt);
	else
		dwidth = 150;

	if (edit_length > 0) {
		Editfield = XtNameToWidget (Dialog, "value");
		XtSetArg (args[0], XtNfont, &font);
		XtSetArg (args[1], XtNrightMargin, &right_margin);
		XtSetArg (args[2], XtNleftMargin, &left_margin);
		XtGetValues (Editfield, args, 3);
		width = right_margin + left_margin +
			CHAR_WIDTH (font) * edit_length;

		XtSetArg (args[0], XtNwidth, width);
		XtSetValues (Editfield, args, 1);

		dwidth = max (dwidth, width);
	}

/* Position Dialog box in center of application */
	XtSetArg (args[0], XtNwidth, &twidth);
	XtSetArg (args[1], XtNheight, &theight);
	XtGetValues (Board, args, 2);

	XtTranslateCoords (Board, (Position) (twidth - dwidth) / 2,
				     (Position) (theight - 58) / 2, &x, &y);
	
	XtMoveWidget (Popupshell, x, y);
	XtPopup (Popupshell, XtGrabExclusive);

	dialog_status = DIALOG_UNDECIDED;

	do {
		XtAppNextEvent (app_con, &event);
		XtDispatchEvent (&event);
	} while (dialog_status == DIALOG_UNDECIDED);

	if (input_buffer != NULL)
		strcpy (input_buffer, XawDialogGetValueString(Dialog));

	XtPopdown (Popupshell);
	XtDestroyWidget (Dialog);
	
	if (dialog_status == DIALOG_CANCEL) {
		message ("Operation cancelled.");
		return False;
	}

	return True;
}


/*
 * Dialog value box Return Key redefinition.
 */
static XtActionsRec actions[] = {
	{ "DialogOk", 	(void (*)()) DialogOk }
};

void init_dialog ()
{
	XtAppAddActions (XtWidgetToApplicationContext (Toplevel),
			 actions, XtNumber (actions));
}
