/*
 * Init.c
 *
 * Icon declarations followed by Initialised tile structures.
 * Initialisation routines.
 */

#include "stratego.h"

#include "icons/free.h"
#include "icons/free_s.h"
#include "icons/free_mask.h"

#include "icons/enemy.h"
#include "icons/enemy_s.h"
#include "icons/enemy_mask.h"

#include "icons/flag.h"
#include "icons/flag_s.h"
#include "icons/flag_mask.h"

#include "icons/marshal.h"
#include "icons/marshal_s.h"
#include "icons/marshal_mask.h"

#include "icons/general.h"
#include "icons/general_s.h"
#include "icons/general_mask.h"

#include "icons/colonel.h"
#include "icons/colonel_s.h"
#include "icons/colonel_mask.h"

#include "icons/major.h"
#include "icons/major_s.h"
#include "icons/major_mask.h"

#include "icons/captain.h"
#include "icons/captain_s.h"
#include "icons/captain_mask.h"

#include "icons/lieutenant.h"
#include "icons/lieutenant_s.h"
#include "icons/lieutenant_mask.h"

#include "icons/sergeant.h"
#include "icons/sergeant_s.h"
#include "icons/sergeant_mask.h"

#include "icons/miner.h"
#include "icons/miner_s.h"
#include "icons/miner_mask.h"

#include "icons/scout.h"
#include "icons/scout_s.h"
#include "icons/scout_mask.h"

#include "icons/spy.h"
#include "icons/spy_s.h"
#include "icons/spy_mask.h"

#include "icons/bomb.h"
#include "icons/bomb_s.h"
#include "icons/bomb_mask.h"

#include "icons/boat_cursor.h"
#include "icons/boat_cursor_mask.h"

#include "icons/board_cursor.h"
#include "icons/board_cursor_mask.h"

#include "icons/account_cursor.h"
#include "icons/account_cursor_mask.h"

#define TILE_ENTRY(name, value, count)	\
	{ value, "name", name/**/_bits, name/**/_s_bits, name/**/_mask_bits, \
	  (Pixmap) 0, (Pixmap) 0, (Cursor) 0, \
	  name/**/_x_hot, name/**/_y_hot, name/**/_width, name/**/_height, \
	  count, count }

#define CURSOR_ENTRY(name) \
	{ 0, NULL, name/**/_bits, NULL, name/**/_mask_bits, \
	  (Pixmap) 0, (Pixmap) 0, (Cursor) 0, \
	  name/**/_x_hot, name/**/_y_hot, name/**/_width, name/**/_height, 0,0 }

TILE tiles[MAX_TILES + MAX_CURSORS] = {
	TILE_ENTRY (free,       FREE,       1),
	TILE_ENTRY (enemy,      ENEMY,      0),
	TILE_ENTRY (flag,       FLAG,       1),
	TILE_ENTRY (marshal,    MARSHAL,    1),
	TILE_ENTRY (general,    GENERAL,    1),
	TILE_ENTRY (colonel,    COLONEL,    2),
	TILE_ENTRY (major,      MAJOR,      3),
	TILE_ENTRY (captain,    CAPTAIN,    4),
	TILE_ENTRY (lieutenant, LIEUTENANT, 4),
	TILE_ENTRY (sergeant,   SERGEANT,   4),
	TILE_ENTRY (miner,      MINER,      5),
	TILE_ENTRY (scout,      SCOUT,      8),
	TILE_ENTRY (spy,        SPY,        1),
	TILE_ENTRY (bomb,       BOMB,       6),
	CURSOR_ENTRY (boat_cursor),
	CURSOR_ENTRY (board_cursor),
	CURSOR_ENTRY (account_cursor)
};

/* Create tile pixmaps and cursors on board window */
void init_icons ()
{
	Display *display = XDISPLAY;
	Pixmap pixmap_mask;
	XColor black_color, white_color;
	TILE *tile;
	int i;

	XLookupColor (display, DefaultColormap (display, XSCREEN), "black",
		      &black_color, &black_color);
	XLookupColor (display, DefaultColormap (display, XSCREEN), "white",
		      &white_color, &white_color);

        for (i = 0; i < MAX_TILES + MAX_CURSORS; i++) {
                tile = &tiles[i];
                tile->pixmap = XCreateBitmapFromData (display, XROOTWINDOW,
                                        tile->bitmap,
                                        tile->width, tile->height);
                pixmap_mask  = XCreateBitmapFromData (display, XROOTWINDOW,
                                            tile->bitmap_mask,
                                            tile->width, tile->height);

                tile->cursor = XCreatePixmapCursor (display, tile->pixmap,
                                        pixmap_mask,
                                        &black_color, &white_color,
                                        tile->hotspot_x, tile->hotspot_y);
                XFreePixmap (display, pixmap_mask);
                XFreePixmap (display, tile->pixmap);

		if (tile->name == NULL)
			continue;		/* Cursor entry */

                tile->pixmap = XCreateBitmapFromData (display, XROOTWINDOW,
                                        tile->bitmap,
                                        tile->width, tile->height);
                tile->stippled_pixmap = XCreateBitmapFromData (display,
						XROOTWINDOW,
                                        	tile->stippled_bitmap,
                                        	tile->width, tile->height);
        }

	XFreeCursor (display, tiles[FREE].cursor);
	tiles[FREE].cursor = tiles[BOARD_CURSOR].cursor;
}

void init_board ()
{
	int y, x;

	for (y = MAX_ROWS - 1; y >= 0; y--)
		for (x = MAX_COLS - 1; x >= 0; x--)
			board[y][x].owner = O_NOBODY;

	board[4][2].value = board[4][3].value = WATER;
	board[5][2].value = board[5][3].value = WATER;

	board[4][6].value = board[4][7].value = WATER;
	board[5][6].value = board[5][7].value = WATER;
}
