/*
 * Register.c
 */

#include "stratego.h"

/*
 * Game Status Register structure definition.
 */
typedef struct s_game_status {
	int  count;		/* # times position has been registered */
	char board_layout[MAX_ROWS * MAX_COLS + 1];
	struct s_game_status *next;
} GameStatusRec, *GameStatus;

static GameStatus status_list = NULL;

#define BoardValue(y, x)	(board[(y)][(x)].owner * MAX_TILES + \
				 board[(y)][(x)].value + ' ')

/*
 * Prints remise messages. Confirms for actual remise move.
 */
static Boolean RemiseSituation(player)
int player;
{
	static char header[] = "Remise Warning!\nThis board setup has occurred twice before.\n";
	char msg_buf[250];

	if (player == O_ENEMY) {
		sprintf (msg_buf, "%sI hereby declare a cease fire!", header);
		(void) DialogBoxResult (msg_buf, NULL, 0, NO_CANCEL);
		return REMISE_MOVE;
	}

	sprintf (msg_buf, "%sPlaying this move will effectuate a remise.",
		 header);

	if (DialogBoxResult(msg_buf, NULL, 0, ADD_CANCEL) == False)
		return UNDO_MOVE;

	return REMISE_MOVE;
}

/*
 * Registers a board setup. Checks if position has occurred before.
 *
 * Issues a warning if position has been registered before
 * Ends game if position has been registers twice already.
 */
int register_board (player)
int player;				/* The one who is moving */
{
	GameStatus new, ptr;
	int y, x;

	new = XtNew(GameStatusRec);
	for (y = 0; y < MAX_ROWS; y++)
		for (x = 0; x < MAX_COLS; x++)
			new->board_layout[y * MAX_ROWS + x] = BoardValue(y, x);
	
	new->board_layout[MAX_ROWS * MAX_COLS] = '\0';

	for (ptr = status_list; ptr != NULL; ptr = ptr->next) {
		if (!strcmp (ptr->board_layout, new->board_layout)) {
			XtFree ((char *)new);

			if (++ptr->count >= 3)
				return RemiseSituation(player);

			(void) DialogBoxResult (
		      "Remise Warning!\nThis board setup has occurred before!",
				NULL, 0, NO_CANCEL);
			break;
		}
	}

	if (ptr == NULL) {
		new->count = 1;
		new->next = status_list;
		status_list = new;
	}

	return DO_MOVE;
}

/*
 * Registers a tile hit. Invalidates all previous positions.
 */
void register_hit ()
{
	GameStatus ptr, next;

	for (ptr = status_list; ptr != NULL; ptr = next) {
		next = ptr->next;
		XtFree ((char *)ptr);
	}

	status_list = NULL;
}
